// Notes.java

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class Notes extends JFrame {

	private JButton openNoteBtn, addNoteBtn, delNoteBtn, closeBtn;
	
	private JList list;
	private DefaultListModel listModel;
	private LinkedList<String> filenames, openFiles;
	
	private JMenuBar menuBar;
	private JMenu helpMenu, optionsMenu;
	private JMenuItem aboutItem, settingsItem;
	
	private final boolean debugMode = true;
	private static final int MAX_NOTES = 256;
	private final float version = 0.2f;
	
	private final String designFile = "design.txt";
	
	private class MyButtonListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			String cmd = e.getActionCommand();
			
			if(cmd == addNoteBtn.getText())
				addNote();
			else if(cmd == delNoteBtn.getText())
				delNote();
			else if(cmd == openNoteBtn.getText())
				openNote();
			else if(cmd == closeBtn.getText())
				System.exit(0);
		}
	}
	
	private class NoteFrm extends JFrame {
	
		private JTextField noteName;
		private JTextArea noteArea;
		private boolean isNew;
		private String givenFilename;
	
		public NoteFrm() {
			super("Note");
			setLayout(new BorderLayout());
			setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
//			setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
/*			addWindowListener(new WindowAdapter() {
				public void windowClosing(ActionEvent e) {
					close();
				}
			});*/
			
			JPanel upperPanel = new JPanel();
			upperPanel.setLayout(new FlowLayout());
			getContentPane().add(upperPanel, BorderLayout.NORTH);
			
			JLabel titleLbl = new JLabel("Title:");
			upperPanel.add(titleLbl);
						
			noteName = new JTextField("", 22);
			upperPanel.add(noteName);
			
			noteArea = new JTextArea(10, 10);		
			noteArea.setLineWrap(true);
			JScrollPane scrollPane = new JScrollPane(noteArea);
			getContentPane().add(scrollPane, BorderLayout.CENTER);
						
			JPanel lowerPanel = new JPanel();
			lowerPanel.setLayout(new FlowLayout());
			getContentPane().add(lowerPanel, BorderLayout.SOUTH);
			
			JButton saveBtn = new JButton("Save");
			saveBtn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {				
					if(noteName.getText().length() < 4 || noteArea.getText().length() < 4)
						Utils.error("Ugly title. Please write something useful!");
					else									
						saveNote();
				}
			});
			lowerPanel.add(saveBtn);
			
			JButton closeBtn = new JButton("Close");
			closeBtn.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					close();
				}
			});
			lowerPanel.add(closeBtn);
						
			setSize(300, 300);			
			setResizable(false);
			
			Point cp = Utils.getCenter(getWidth(), getHeight());
			setLocation(cp);
			
			isNew = true;
		}
		
		public void loadNote(String filename) {
			isNew = false;		
			givenFilename = filename;
			
			try {
				FileReader fr = new FileReader(filename);
				BufferedReader br = new BufferedReader(fr);
				
				noteName.setText(br.readLine());
				
				String line = br.readLine();
				while(line != null) {
					noteArea.append(line + "\n");
					line = br.readLine();
				}
				
				fr.close();
				
				setTitle("Note - " + noteName.getText());
			}
			catch(IOException e) {
				Utils.error("Can't read file: " + filename);
			}
		}
		
		private String getFilename() {
			int i;
			String name = "notes/note0.txt";
			
			for(i=0; i<MAX_NOTES; i++) {
				name = "notes/note" + i + ".txt";
				if(!Utils.fileExists(name))
					break;
			}
			
			return name;
		}
		
		private void saveNote() {		
			String filename;
			
			if(isNew)
				filename = getFilename();
			else
				filename = givenFilename;
			
			try {
				FileWriter fw = new FileWriter(filename);
				BufferedWriter bw = new BufferedWriter(fw);
				
				bw.write(noteName.getText());
				bw.newLine();
				bw.write(noteArea.getText());
				bw.flush();
				
				fw.close();
				
			} catch(IOException e) {
				Utils.error("Unable to write file: " + filename);
			}		
			
			close();
		}
		
		private void close() {
			reenable();
			setVisible(false);
			
			if(!isNew) {
				int i;
				for(i=0; i<openFiles.size(); i++)
					if(openFiles.get(i).equals(givenFilename))
						break;
				
				openFiles.remove(i);
			}
		}
	
	}
	
	private void openNote() {	
		int i;
		
		if(listModel.firstElement() == "Empty")
			return;
		
		String itsFilename = filenames.get(list.getSelectedIndex());
		
		for(i=0; i<openFiles.size(); i++)
			if(openFiles.get(i).equals(itsFilename))
				return;
	
		NoteFrm note = new NoteFrm();
//		setEnabled(false);
		note.setVisible(true);		
		note.loadNote(itsFilename);
		debug("Opening note: " + itsFilename);
		
		openFiles.add(itsFilename);
	}
	
	private void addNote() {
		debug("Adding note...");
		NoteFrm newNote = new NoteFrm();
//		setEnabled(false);
		newNote.setVisible(true);
	}
	
	private void delNote() {
		if(listModel.size() > 0) {
			if(listModel.firstElement() != "Empty") {
			
				String filename = filenames.get(list.getSelectedIndex());
				File file = new File(filename);
				file.delete();
				
				debug("Deleting " + filename);
				
				filenames.remove(list.getSelectedIndex());				
				listModel.remove(list.getSelectedIndex());				
				
				if(listModel.size() == 0)
					listModel.addElement("Empty");
			}
		}
		else
			listModel.addElement("Empty");
		
		list.setSelectedIndex(0);
	}
	
	private void reenable() {
//		setEnabled(true);
		updateList();
	}
	
	private	void addBtn(JButton button, JPanel pane) {
		button.addActionListener(new MyButtonListener());
		pane.add(button);
	}

	Notes() {
		super("Notes");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLayout(new BorderLayout());
		
		debug("Starting notes...");
		
		if(Utils.fileExists(designFile)) {
			String lafName = UIManager.getLookAndFeel().getID();
			
			try {
				FileReader fr = new FileReader(designFile);
				BufferedReader br = new BufferedReader(fr);
				
				lafName = br.readLine();
				
				fr.close();
				
				UIManager.setLookAndFeel(lafName);
			}
			catch(IOException e) {
				Utils.error("Can't read out design!");
			}
			catch(ClassNotFoundException cnfe) {
				Utils.error("Unable to find class!");
			}
			catch(Exception e) {
				Utils.error("This is odd!");
			}
		}
		
		menuBar = new JMenuBar();		
		setJMenuBar(menuBar);
		
		helpMenu = new JMenu("Help");
		menuBar.add(helpMenu);
		
		aboutItem = new JMenuItem("About Notes...");
		aboutItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(null,
					"Notes - small java based reminder app\n" +
					"Version: " + String.valueOf(version) + "\n" +
					"2006 by Andre Schnabel\n" +
					"License: GPL\n" +
					"Visit www.semler-service.de/andre/ for more stuff!",
					"About Notes...",
					JOptionPane.INFORMATION_MESSAGE);
			}
		});
		helpMenu.add(aboutItem);
		
		optionsMenu = new JMenu("Options");
		menuBar.add(optionsMenu);
		
		settingsItem = new JMenuItem("Settings");
		settingsItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				showSettings();
			}
		});
		optionsMenu.add(settingsItem);
		
		filenames = new LinkedList<String>();
		openFiles = new LinkedList<String>();
				
		JLabel titleLbl = new JLabel("List of notes:");
		getContentPane().add(titleLbl, BorderLayout.NORTH);

		listModel = new DefaultListModel();
		listModel.addElement("Empty");
		
		list = new JList(listModel);
		list.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				openNote();
			}
		});
		list.setSelectedIndex(0);
		list.setLayoutOrientation(JList.VERTICAL);
		list.setVisibleRowCount(-1);
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);		
		JScrollPane listScroll = new JScrollPane(list);
		getContentPane().add(listScroll, BorderLayout.CENTER);
		
		updateList();
		
		JPanel lowerPanel = new JPanel();
		lowerPanel.setLayout(new GridLayout(2, 2));
		getContentPane().add(lowerPanel, BorderLayout.SOUTH);
		
		openNoteBtn = new JButton("Open");
		openNoteBtn.setToolTipText("Open the selected note");
		addBtn(openNoteBtn, lowerPanel);
		
		addNoteBtn = new JButton("Add");
		addNoteBtn.setToolTipText("Add a new note");
		addBtn(addNoteBtn, lowerPanel);
		
		delNoteBtn = new JButton("Delete");
		delNoteBtn.setToolTipText("Delete the selected note");
		addBtn(delNoteBtn, lowerPanel);
		
		closeBtn = new JButton("Close");
		closeBtn.setToolTipText("Close this app");
		addBtn(closeBtn, lowerPanel);
		
		pack();
		
		setSize(200, 300);		
		setResizable(false);
		
		Point cp = Utils.getCenter(getWidth(), getHeight());
		setLocation(cp);
		
		debug("Notes started!");
	}
	
	private void showSettings() {
		final JFrame settingsFrm = new JFrame("Settings");
		LookAndFeel currentLaf = UIManager.getLookAndFeel();
		String curLafName = currentLaf.getName();
		int curLafIndex = 0;
		
		settingsFrm.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		settingsFrm.setLayout(new GridLayout(2, 1));
		
		UIManager.LookAndFeelInfo[] lafInfos = UIManager.getInstalledLookAndFeels();
		String[] lafs;
		
		lafs = new String[lafInfos.length];
		int i;
		for(i=0; i<lafs.length; i++) {
			lafs[i] = lafInfos[i].getName();
			
			if(lafs[i] == curLafName)
				curLafIndex = i;
		}
		
		final JComboBox lafSelection = new JComboBox(lafs);
		lafSelection.setSelectedIndex(curLafIndex);
		settingsFrm.getContentPane().add(lafSelection);
		
		JButton okBtn = new JButton("Ok");
		okBtn.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				applySettings(lafSelection.getSelectedIndex());
				settingsFrm.setVisible(false);
				JOptionPane.showMessageDialog(null,
					"Restart the application for any visible effect...", "Info",
					JOptionPane.INFORMATION_MESSAGE);
			}
		});		
		settingsFrm.getContentPane().add(okBtn);
		
		settingsFrm.pack();
		
		Point cp = Utils.getCenter(settingsFrm.getWidth(), settingsFrm.getHeight());
		settingsFrm.setLocation(cp);
		
		settingsFrm.setVisible(true);
		settingsFrm.setAlwaysOnTop(true);
	}
	
	private void applySettings(int selLafIndex) {
		
		UIManager.LookAndFeelInfo[] lafInfos = UIManager.getInstalledLookAndFeels();
		String lafClassName = lafInfos[selLafIndex].getClassName();
		
		try {
			UIManager.setLookAndFeel(lafClassName);
		}
		catch(ClassNotFoundException e) {
			Utils.error("Can't find class!");
		}
		catch(Exception err) {
			Utils.error("Ugly shit...");
		}
		
//		SwingUtilities.updateComponentTreeUI(parent);
		
		writeDesignSettings(lafClassName);
	}
	
	private void writeDesignSettings(String lafClassName) {
		try {
			FileWriter fw = new FileWriter(designFile);
			BufferedWriter bw = new BufferedWriter(fw);
			
			bw.write(lafClassName);
			
			bw.flush();
			fw.close();
		}
		catch(IOException e) {
			Utils.error("Unable to write design settings!");
		}
	}
	
	private void updateList() {
		int i;
		String filename = "notes/note0.txt", title;
		int numNotes = 0;
		
		listModel.removeAllElements();
		filenames.clear();
		
		for(i=0; i<MAX_NOTES; i++) {
			filename = "notes/note" + i + ".txt";
			if(Utils.fileExists(filename)) {
				try {
					FileReader fr = new FileReader(filename);
					BufferedReader br = new BufferedReader(fr);
					title = br.readLine();
					listModel.addElement(title);
					filenames.add(filename);
					fr.close();
					numNotes++;
				}
				catch(IOException e) {
					Utils.error("Failed to read out of file: " + filename);
				}
			}			
		}
		
		list.setSelectedIndex(0);
		
		if(numNotes == 0)
			listModel.addElement("Empty");
	}
	
	private void debug(String message) {
		if(debugMode)
			System.out.println(message);
	}
	
	public static void main(String[] args) {
		Notes instance = new Notes();
		instance.setVisible(true);
	}
}
